//===========================================================
// MD6602
//-----------------------------------------------------------
// File Name   : hardware.c
// Description : Sample Program for MD6602
//-----------------------------------------------------------
// History :
// Rev.00 2016.03.31  1ch Buck-Converter  First Release
// PCB: PDA50014+ChewingGum_MD6602
//-----------------------------------------------------------
// Copyright (C) 2016 SANKEN ELECTRIC CO.,LTD.
//===========================================================

#include "hardware.h"
SI16 Vref_SS = 0; //Vref_SoftStart
SI16 Vref_Tar = 0xfdd2; //Vref_Target(-:1.8V)

//---------------------------------------------------------------------
// Wait
//---------------------------------------------------------------------
void wait(UI32 cycle)
{
    volatile UI32 count;
    for (count = 0; count < cycle; count++);
}

//---------------------------------------------------------------------
// Configure Clock
//---------------------------------------------------------------------
void Config_Clock(void)
{
    // Use Xtal
#if 0
    if (1)
    {
        CLKCFG1 = CLKCFG1 | 0x02; // set OSCE
        wait(5000);
//        wait(5);
        CLKCFG1 = CLKCFG1 | 0x04; // set CLKSRC
        while(1) {if (CLKCFG1 & 0x04) break;}
    }
#endif
    // Use PLL
    if (1)
    {
        CLKCFG0 = CLKCFG0 | 0x40; // set PLLEN
        wait(5000);
//        wait(5);
        CLKCFG0 = CLKCFG0 | 0x20; // set CLKPLL
        while(1) {if (CLKCFG0 & 0x20) break;}
    }
    // Set Divider
    if (1)
    {
        CLKCFG0 = CLKCFG0 | 0x0f; // set DIV1, DIV0
        while(1) {if ((CLKCFG0 & 0x0f) == 0x0f) break;}
    }
    // Enable Module Clock
    MCLKE0 = 0x1f; // TIM0, SPI, I2C, UART, GPIO
    MCLKE1 = 0x33; // DSP1, DSP0, PWMPLL, PWM
    MCLKE2 = 0x71; // ADC2, ADC1, ADC0, DAC
    MCLKE3 = 0xbf; // SPR, AMP1, AMP0, CMP3, CMP2. CMP1. CMP0
    MCLKE4 = 0xb3; // CMPLUT, AMP3, AMP2, CMP5, CMP4
}

//---------------------------------------------------------------------
// Initialize Tick
//---------------------------------------------------------------------
void Init_Tick(void)
{
    // Configure Timer0
    // TMR0 Period: 7.5us = 25ns * 300 * (1)
    // SoftStart Period: 7.5us * 558 = 4.185ms
    TMCR0 = 0x2A;   //[0xFA04]
                    // 0---_----: Underflow INT. Disabled
                    // -0--_----: Overflow INT. Disabled
                    // --1-_----: Compare Match A INT. Enabled
                    // ---0_----: Compare Match B INT. Disabled
                    // ----_1---: Compare Match A Event Out Enabled
                    // ----_-0--: Compare Match B Event Out Disabled
                    // ----_--1-: Timer Auto Clear Enabled
                    // ----_---0: Timer Clear at Compare Match A
//    TCMPAL0 = (300 & 0x00ff); //Lower 8bit Trans.
//    TCMPAH0 = (300 >> 8);     //Upper 8bit Trans.
    TCMPAL0 = (800 & 0x00ff); //Lower 8bit Trans.
    TCMPAH0 = (800 >> 8);     //Upper 8bit Trans.

    TMOD0 = 0x60;   //[0xFA00]
                    // 0---_----: Timer Disabled
                    // -1--_----: Timer Interrupt Master Enabled
                    // --1-_----: Compare Match A Enabled
                    // ---0_----: Compare Match B Disabled
                    // ----_-000: Prescaler is 1/1

    TOACR0 |= 0x30; //[0xFA31]
                    // --11_----: Toggle Output at Clearing TCNT
}

//---------------------------------------------------------------------
// Start Timer
//---------------------------------------------------------------------
void Start_TMR(void)
{
    // Start Timer0
    TMOD0 |= 0x80; //[0xFA00]
                   // 1---_----: Timer Enabled
}

//---------------------------------------------------------------------
// ISR for Tick 100us
//---------------------------------------------------------------------
void ISR_Tick_10us(void)
{

//Function Check GPIO16 Start//
Toggle_GPIO16();
//Function Check GPIO16 End//

    // Clear the TMR0CMA Flag
    TMSR0 = 0x01;

    // Clear the INT.Flag
    INTFLG2 = 0x20;

    if(Vref_SS == Vref_Tar)      // End SS Ope. when Ref. is equal to Tar.
    {
//        INTENA2 = INTENA2 & 0xdf;
        INTMST  = INTMST  & 0xfe;
    }
    else if(Vref_SS > Vref_Tar)  // Dec. Ref Val. when Ref. is greater than Tar.
    {
        Vref_SS = Vref_SS - 1;
        ADOL00 = Vref_SS ;       //
        ADOH00 = Vref_SS >> 8;   //
    }
    else                         // Inc. Ref Val. when Ref. is smaller than Tar.
    {
        Vref_SS = Vref_SS + 1;
        ADOL00 = Vref_SS ;       //
        ADOH00 = Vref_SS >> 8;   //
    }
}

//---------------------------------------------------------------------
// Initialize INTC
//---------------------------------------------------------------------
void Init_INTC(void)
{
    // Enable INTC
    // Timer0 Vec=21(0xab) Low
    INTCFG2 = INTCFG2 & 0xdf; //[0xB6]INT. Sensitivity
                              // INT.Vec.21 is Low level
    INTLVL2 = INTLVL2 | 0x20; //[0xAE]
                              // INT.Vec.21 is High Prio. Level
    INTENA2 = INTENA2 | 0x20; //[0xA6]
                              // INT.Vec.21 Enabled(TMR0)
    INTMST  = INTMST  | 0x01; //[0x9C]
                              // x---_----: High Prio. INT. Flag
                              // -x--_----: Low Prio. INT. Flag
                              // ----_---1: Permit INT. Master Enable
}

//---------------------------------------------------------------------
// Initialize GPIO
//---------------------------------------------------------------------
void Init_GPIO(void)
{
    PFS0 = 0x30; //[0xFE00] = 0x30
                 // --11_----: TIOA0 Output Func. is Selected for GPIO02
    PFS1 = 0x03; //[0xFE01] = 0x03
                 // ----_--1-: (GPIO10)PWM0H
                 // ----_---1: (GPIO11)PWM0L
    PFS2 = 0x55; //[0xFE02] = 0x55
                 // -1--_----: (GPIO26)ANEX6
                 // ---1_----: (GPIO24)ANEX4
                 // ----_-1--: (GPIO22)ANEX2
                 // ----_---1: (GPIO20)ANEX0
    PFS3 = 0xA2; //[0xFE03] = 0xA2
                 // 1---_----: (GPIO37)ANEX15 OPA3 NEG
                 // --1-_----: (GPIO35)ANEX13 OPA3 POS
                 // ----_--1-: (GPIO31)ANEX9  DAC0 OUT
//---------------------------------------------------------------------
    PDD0 = 0x00;  // [0xFE04] 
    PDD1 = 0xC3;  // [0xFE05] 1100_0011
                  // ---- ---1: GPIO10(output)
                  // ---- --1-: GPIO11(output)
                  // -1-- ----: GPIO16(output)
                  // 1--- ----: GPIO17(output)
    PDD2 = 0x00;  // [0xFE06] 
    PDD3 = 0x02;  // [0xFE07] 
                  // ----_--1-: (GPIO31)ANEX9  DAC0 OUT
    PDD4 = 0x00;  // [0xFE44] 
    PDD5 = 0x00;  // [0xFE45] 

    PDR0 = 0x00;  // [0x90] 
    PDR1 = 0x00;  // [0x98] 
    PDR2 = 0x00;  // [0xB0] 
    PDR3 = 0x00;  // [0xC0] 
    PDR4 = 0x00;  // [0xA0] 
    PDR5 = 0x00;  // [0xA8] 

    PPU0 = 0xFF;  // [0xFE08] 1111_1111
    PPD1 = 0xFF;  // [0xFE0D] 1111_1111: Pull Down ALL MOSFET
    PPU2 = 0xFF;  // [0xFE0A] 1111_1111
    PPU3 = 0xFF;  // [0xFE0B] 1111_1111
    PPU4 = 0xFF;  // [0xFE48] 0011_1111
    PPD5 = 0xFF;  // [0xFE4D] 0000_1111
//---------------------------------------------------------------------
}

//---------------------------------------------------------------------
// Initialize DSAC
//---------------------------------------------------------------------
void Init_DSAC(void)
{
    //--------------------------
    // ch0 (Tiny-DSP0_R0 => BUF_A0)
    //--------------------------
    DSASRC0  = 0xC4; //[0xF882] SFR Source Address = 0xC4 (DSP0_R0)
    DSADST0  = 0xE4; //[0xF883] SFR Dest. Address = 0xE4 (BUF_A0)
    DSACNTA0 = 0x13; //[0xF880]
                     // 0---_----: DSA CH Disable
                     // -00-_----: 1 word Trans.(Once)
                     // ---1_0011: Trig.#19=0x13(TinyDSP0 EVT0)
    DSACNTB0 = 0x80; //[0xF881]
                     // 1---_----: Word Data Access is 1word(16bit)
    //--------------------------
    // ch1 (10bitADC0_ch0 => Tiny-DSP0_R4)
    //--------------------------
    DSASRC1  = 0x99; //[0xF886] SFR Source Address = 0x99 (AD00)
    DSADST1  = 0xCC; //[0xF887] SFR Dest. Address = 0xCC (DSP0_R4) 
    DSACNTA1 = 0x04; //[0xF884]
                     // 0---_----: DSA CH Disable
                     // -00-_----: 1 word Trans.(Once)
                     // ---0_0100: Trig.#04=0x04(ADC0 EVT)
    DSACNTB1 = 0x80; //[0xF885]
                     // 1---_----: Word Data Access is 1word(16bit)
    //--------------------------
    // ch2 (Tiny-DSP0_R3 => 10bitDAC0)
    //--------------------------
/**/
    DSASRC2  = 0xC7; //[0xF88A] SFR Source Address = 0xC4 (DSP0_R3)
    DSADST2  = 0x96; //[0xF88B] SFR Dest. Address = 0x96 (DAC0)
    DSACNTA2 = 0x34; //[0xF888]
                     // 0---_----: DSA CH Disable
                     // -00-_----: 2 byte Trans.(twice)
                     // ---1_0100: Trig.#20=0x14(TinyDSP0 EVT1)
    DSACNTB2 = 0x00; //[0xF889] Address stays at same value Dest. And Source
                     // 0---_----: Word Data Access is 1byte(8-bit)

}

//---------------------------------------------------------------------
// Start DSAC (ch0,ch1)
//---------------------------------------------------------------------
void DSAC_start(void)
{
    DSACNTA0 |= 0x80; // start DSAC ch0
    DSACNTA1 |= 0x80; // start DSAC ch1
    DSACNTA2 |= 0x80; // start DSAC ch2
}

//---------------------------------------------------------------------
// Stop DSAC (ch0,ch1)
//---------------------------------------------------------------------
void DSAC_stop(void)
{
    DSACNTA0 &= 0x7F; // stop DSAC ch0
    DSACNTA1 &= 0x7F; // stop DSAC ch1
    DSACNTA2 &= 0x7F; // stop DSAC ch2
}

//---------------------------------------------------------------------
// Initialize ADC0 (Input:ANEX0,VFB)
//---------------------------------------------------------------------
void Init_ADC0(void)
{
    ADC0     = 0x35; //[0xF000]
                     // 0---_----: CH0 Update Threshold Disable
                     // -0--_----: Burst Until Sequence End Mode
                     // --1-_----: Fast Mode (CLKADCn_133 / 2)
                     // ---1_0101: PWM0-EVENT0 (pulse)
    ADSM0    = 0x00; //[0xF001] Sample Mode = Single
    ADSL0    = 0x01; //[0xF002] CH SEQUENCE : ch0 -> END
    ADSH0    = 0x00; //[0xF003] CH SEQUENCE : NO Use
    ADTRGL0  = 0x01; //[0xF004] Ch0 EVENT Output
    ADTRGH0  = 0x00; //[0xF005] NO EVENT Output
    ADOL00   = 0x00; //[0xF010] ADC0_ch0 OFFSET,0x0000(0)
    ADOH00   = 0x00; //[0xF011]
//    ADOL00   = 0xd2; //[0xF010] ADC0_ch0 OFFSET,0xfdd2(-558,1.8V)
//    ADOH00   = 0xfd; //[0xF011]
    ADEN0    = 0x01; //[0xF007] ADC0 Enabled

    // -----------------------------------------
    wait(10);  // wait for ?sec (10*?sec)
    // -----------------------------------------
}

//---------------------------------------------------------------------
// Initialize ADC1 (Input:ANEX4,ch0)
//---------------------------------------------------------------------
void Init_ADC1(void)
{
    ADC1     = 0x31; //[0xF080]
                     // 0---_----: CH0 Update Threshold Disable
                     // -0--_----: Burst Until Sequence End Mode
                     // --1-_----: Fast Mode (CLKADCn_133 / 2)
                     // ---1_0001: TIMER0-CM0 (pulse)
    ADSM1    = 0x00; //[0xF081]Sample Mode = Single
    ADSL1    = 0x01; //[0xF082]CH SEQUENCE : ch0 -> END
    ADSH1    = 0x00; //[0xF083]CH SEQUENCE : NO Use
    ADTRGL1  = 0x01; //[0xF084]Ch0 EVENT Output
    ADTRGH1  = 0x00; //[0xF085]NO EVENT Output
    ADEN1    = 0x01; //[0xF087]ADC1 Enabled
}

//---------------------------------------------------------------------
// Initialize ADC2 (Input:ANEX2,ch1)
//---------------------------------------------------------------------
void Init_ADC2(void)
{
    ADC2 = 0x31;       //[0xF100]Up-th Disable,Burst Mode,Fast Mode,pwm0_event0
                       // 0---_----: CH0 Update Threshold Disable
                       // -0--_----: Burst Until Sequence End Mode
                       // --1-_----: Fast Mode (CLKADCn_133 / 2)
                       // ---1_0001: TIMER0-CM0 (pulse)
    ADSL2 = 0x02;      //[0xF102] use channel 1
    ADSH2 = 0x00;      //[0xF103]
    ADTRGL2 = 0x02;    //[0xF104] Ch1 EVENT Output
    ADTRGH2 = 0x00;    //[0xF105]
    ADUPTH0L2 = 0x00;  //[0xF109] unused
    ADUPTH0H2 = 0x00;  //[0xF10A]
//    ADOL02    = 0x47;  //[0xF111] ADC2_ch0 OFFSET,0xf747(-2233)
//    ADOH02    = 0xf7;  //[0xF112]
    ADEN2 = 0x01;      //[0xF107]
                       // ----_---1: ADC2 Enabled
    ADI |= 0x00;       //[0xF2] AD INT. Disabled
                       // -0--_----: ADC2 INT. Disabled
                       // --0-_----: ADC1 INT. Disabled
                       // ---0_----: ADC0 INT. Disabled
                       // ----_-0--: ADC2 INT. Flag Clear, when write 1
                       // ----_--0-: ADC1 INT. Flag Clear, when write 1
                       // ----_---0: ADC0 INT. Flag Clear, when write 1

    // -----------------------------------------
    wait(10);  // wait for ?sec (10*?sec)
    // -----------------------------------------
}

//---------------------------------------------------------------------
// OPAMP Register Setting
//---------------------------------------------------------------------
void Init_OPAMP(void)
{
    //--------------------------
    //Mix OPAMP3 (for Io_det)
    //--------------------------
    //Config.
    MIXOPA3 = 0xA0; //[0xEF80]
                    // 1---_----: OPAMP Enabled
                    // -0--_----: OPA Output Pin L is Disabled(ANEX19)
                    // --1-_----: OPA Output Pin M is Enabled(ANEX6)
                    // ---0_----: OPA Output Pin N is Disabled(ANEX4)
                    // ----_00--: Input(-): External Pin(ANEX15) is Selected
                    // ----_--00: Input(+): External Pin(ANEX13) is Selected
    //PGA Config.
    MIXPGA3 = 0x00; //[0xEF81]
                    // ----_---0: Select OPAMP func.
}

//---------------------------------------------------------------------
// Initialize DAC
//---------------------------------------------------------------------
void Init_DAC(void)
{
    //--------------------------
    //DAC0 (Duty Monitor)
    //--------------------------
    MIXDAC0 = 0xD0;    //[0xf200]
                       // 1---_----: DAC0 Enabled
                       // -1--_----: Int.Format(0000 00dd dddd dddd)
                       // --01_0000: DAC0 Update at PWM0_EVT0
    DACOUT0 = 0x01;    //[0xf201]
                       // 0---_----: Normal mode
                       // ----_--0-: Not Connected to ANEX16
                       // ----_---1: Connected to ANEX9
    MIXDARD0L = 0x00;  //[0xf202]   0x0000(0)
    MIXDARD0H = 0x00;  //[0xf203]
    DA0 = 0x00;        //[0x96]
    DA0 = 0x00;        //[0x96]

    //--------------------------
    //DAC1 (OVP Threshold,Connected to CMP4_Input(-))
    //--------------------------
    MIXDAC1 = 0xC0;    //[0xf208]
                       // 1---_----: DAC1 Enabled
                       // -1--_----: Int.Format(0000 00dd dddd dddd)
                       // --00_0000: Immed. Update by CPU or DSAC writing
    DACOUT1 = 0x00;    //[0xf209]
                       // 0---_----: Normal mode
                       // ----_--0-: Not Connected to ANEX17
                       // ----_---0: Not Connected to ANEX11
    MIXDARD1L = 0x26;  //[0xf20A]   0x0326(806)(2.6V)
    MIXDARD1H = 0x03;  //[0xf20B]
    DA1 = 0x26;        //[0x95]
    DA1 = 0x03;        //[0x95]

    //--------------------------
    //DAC2 (OCP Threshold,Connected to CMP5_Input(-))
    //--------------------------
    MIXDAC2 = 0xC0;    //[0xF210]
                       // 1---_----: DAC2 Enabled
                       // -1--_----: Int.Format(0000 00dd dddd dddd)
                       // --00_0000: Immed. Update by CPU or DSAC writing
    DACOUT2 = 0x00;    //[0xF211]
                       // 0---_----: Normal mode
                       // ----_--0-: Not Connected to ANEX18
                       // ----_---0: Not Connected to ANEX13
    MIXDARD2L = 0x93;  //[0xF212]   0x0193(403)(1.3V,Io=2.4A)
    MIXDARD2H = 0x01;  //[0xF213]
    DA2 = 0x93;        //[0x92]
    DA2 = 0x01;        //[0x92]

    //--------------------------
    //DAC3 (NOT Used)
    //--------------------------
    MIXDAC3 = 0x40;    //[0xf218]
                       // 0---_----: DAC3 Disabled
                       // -1--_----: Int.Format(0000 00dd dddd dddd)
                       // --00_0000: Immed. Update by CPU or DSAC writing
    DACOUT3 = 0x00;    //[0xf219]
                       // 0---_----: Normal mode
                       // ----_--0-: Not Connected to ANEX19
                       // ----_---0: Not Connected to ANEX15
    MIXDARD3L = 0x00;  //[0xf21A]
    MIXDARD3H = 0x00;  //[0xf21B]
    DA3 = 0x00;        //[0x93]
    DA3 = 0x00;        //[0x93]
}

//---------------------------------------------------------------------
// Comparator Register Setting
//---------------------------------------------------------------------
void Init_COMP(void)
{
    //--------------------------
    //Comparator4(for Vo OVP)
    //--------------------------
    //CMP4 Config.
    MIXCMP4 = 0xC0; //[0xEF80]
                    // 1---_----: Comparator Enabled
                    // -100_0---: Select Input (-): DAC1 Output
                    // ----_-000: Select Input (+): External Pin(ANEX4)
    //CMP4 Func.Sel.
    MIXCMS4 = 0x5B; //[0xED81]
                    // -1--_----: Level Sense Enabled
                    // --0-_----: Level Sense Interrupt Disabled
                    // ---1_----: Level Sense Polarity is High
                    // ----_10--: Level Edge Sense Polarity is Pos. Edge
                    // ----_--11: Level Glitch Filter is 4cyc.
    //--------------------------
    //Comparator5 Configuration(for OCP)
    //--------------------------
    //CMP5 Config.
    MIXCMP5 = 0xC0; //[0xEE00]
                    // 1---_----: Comparator Enabled
                    // -100_0---: Select Input (-): DAC2 Output
                    // ----_-000: Select Input (+): External Pin(ANEX6)
    //CMP5 Func.Sel.
    MIXCMS5 = 0x5B; //[0xEE01]
                    // -1--_----: Level Sense Enabled
                    // --0-_----: Level Sense Interrupt Disabled
                    // ---1_----: Level Sense Polarity is High
                    // ----_10--: Level Edge Sense Polarity is Pos. Edge
                    // ----_--11: Level Glitch Filter is 4cyc.
}

//-------------------------
// Initialize LUT
//-------------------------
void Init_LUT(void)
{
    // LUT0 Control Register
    LUT0CR = 0xC0;   //[0xEC80]: F/F,w/noise_filter
    // LUT0 Output Register 0-7
    LUT0OUT0 = 0x00; //[0xEC88] CMP4 or CMP5
    LUT0OUT1 = 0x00; //[0xEC89]
    LUT0OUT2 = 0xFF; //[0xEC8A]
    LUT0OUT3 = 0xFF; //[0xEC8B]
    LUT0OUT4 = 0xFF; //[0xEC8C]
    LUT0OUT5 = 0xFF; //[0xEC8D]
    LUT0OUT6 = 0xFF; //[0xEC8E]
    LUT0OUT7 = 0xFF; //[0xEC8F]
}

//-------------------------
// Initialize POC
//-------------------------
void Init_POC(void)
{
    //POC Ctrl.Reg.   [0xfd80]
    POCCR0 = 0xC6;    // 1---_----: Hi-Z Enabled
                      // -1--_----: Auto Release Enabled
                      // --00_----: PWM0H/L Output Control Enabled
                      // ----_-110: CMPLUT CH0
    //POC Status.Reg. [0xfd88]
    POCSTS = 0x01;    // ----_---1: PWM0H &L are controlled by POC
}

//---------------------------------------------------------------------
// PWM Register Configuration
//---------------------------------------------------------------------
const UI08 pwm0_data[] = 
{
    0x00, //PWM0CSS    = 0xf904 ; Use Clock Source A
    0x01, //PWM0EVO0   = 0xf905 ; Event0=EVT_MIN
    0x02, //PWM0EVO1   = 0xf906 ; Event1=EVT_MAX
    0x10, //PWM0EVOT   = 0xf907 ; EventT=EVT_VH0
    0x40, //PWM0INTS0  = 0xf908 ; INT_VT
    0x02, //PWM0INTS1  = 0xf909 ; INT_MAX
    0x00, //PWM0INTF   = 0xf90a ; PWMIF
    0x00, //PWM0ACCLR  = 0xf90b ; No effect
    0x00, //PWM0ACSTS  = 0xf90c ; Not accessing a SFR
    0x00, //reserved   = 0xf90d ; 
    0x00, //reserved   = 0xf90e ;
    0x00, //reserved   = 0xf90f ;
    //
    0x18, //CNT0_L     = 0xf910 ; CNT0=0xfc18 (-1000)
    0xfc, //CNT0_H     = 0xf911
    0xc0, //CMP_A0_L   = 0xf912 ; CMP_A0=0x03c0 (960)
    0x03, //CMP_A0_H   = 0xf913
    0x00, //CMP_B0_L   = 0xf914 ; CMP_B0=0x0000 (0) : No Use
    0x00, //CMP_B0_H   = 0xf915
    0x50, //CMP_C0_L   = 0xf916 ; CMP_C0=0x0050 (80)
    0x00, //CMP_C0_H   = 0xf917
    0x50, //CMP_D0_L   = 0xf918 ; CMP_D0=0x0050 (80)
    0x00, //CMP_D0_H   = 0xf919
    0x18, //CMP_MIN0_L = 0xf91a ; CMP_MIN0=0xfc18 (-1000)
    0xfc, //CMP_MIN0_H = 0xf91b
    0xe8, //CMP_MAX0_L = 0xf91c ; CMP_MAX0=0x03e8 (1000)
    0x03, //CMP_MAX0_H = 0xf91d
    0x00, //reserved   = 0xf91e
    0x00, //reserved   = 0xf91f
    //
    0x01, //PWM0CNTMD  = 0xf920 ; 0000_0001: Up-Down Mode
    0x01, //PWM0HCR0   = 0xf921 ; 0000_0001: InitH=L
    0x02, //PWM0LCR0   = 0xf922 ; 0000_0010: InitL=H
    0x06, //PWM0HCR1   = 0xf923 ; 0000_0110: VH1=L,VH0=H
    0x09, //PWM0LCR1   = 0xf924 ; 0000_1001: VL1=H,VL0=L
    //
    0x71, //PWM0MODE    = 0xf925 ; 0111_0001: DTMode0,BufferMode,Both Top & Bot_Timing,PWMMode1
//    0x51, //PWM0MODE    = 0xf925 ; 0101_0001: DTMode0,BufferMode,Top_Timing,PWMMode1
//    0x61, //PWM0MODE    = 0xf925 ; 0110_0001: DTMode0,BufferMode,Bot_Timing,PWMMode1
    0x01, //PWM0RTRG    = 0xf926 ; 0000_0001: Disable Re-Trigger Mode B
    0x04, //PWM0RTRS    = 0xf927 ; 0000_0101: CMP0 Trigger (Event 4): No Use
    0x00, //PWM0RTGC    = 0xf928 ; 0000_0000: NOP
    0x06, //PWM0RTL     = 0xf929 ; 0000_0110: VTH=L,VTL=H
    0x00, //PWM0RTMC    = 0xf92a ; 1000_0000: ReTrig_Mask disable
    0x0a, //PWM0RTMP    = 0xf92b ; 0000_1010: 10cycles(1.25nsec*8*10cycles=100nsec)
    //
    0x18, //BUF_MIN0_L  = 0xf92c ; BUF_MIN0=0xfc18 (-1000)
    0xfc, //BUF_MIN0_H  = 0xf92d ; 
    0xe8, //BUF_MAX0_L  = 0xf92e ; BUF_MAX0=0x03e8 (1000)
    0x03, //BUF_MAX0_H  = 0xf92f ; 
};

//---------------------------------------------------------------------
//PWM0 Configure
//---------------------------------------------------------------------
void pwm0_do_config(void)
{
    __xdata  UI08 * __xdata ptr;

    UI08 i;

    //Register in xdata area
    ptr = &PWM_BASE_0;
    for (i = 0; i < 44; i++)
    {
       while(PWM0ACSTS & 0x01);
       *ptr++ = pwm0_data[i];
    }
    // PWM BUF_A0, BUF_B0, BUF_C0, BUF_D0
    BUF_A0_LH = 0xc0;  // BUF_A3=0x03c0 (960)
    BUF_A0_LH = 0x03;
    BUF_B0_LH = 0x00;  // BUF_B3=0x0000 (0) 
    BUF_B0_LH = 0x00;
    BUF_C0_LH = 0x50;  // BUF_C3=0x0050 (80)
    BUF_C0_LH = 0x00;
    BUF_D0_LH = 0x50;  // BUF_D3=0x0050 (80)
    BUF_D0_LH = 0x00;
}

//---------------------------------------------------------------------
// Initialize PWM
//---------------------------------------------------------------------
void Init_PWM(void)
{
    // Clock Settings
    PWMCSC0 = 0x00; // CLKA = CLKB = PLL2/(2^0)=800MHz (1.25nsec)
    PWMCSC1 = 0x00; // CLKC = CLKD = PLL2/(2^0)=800MHz (1.25nsec)
    PWMENBL = 0x01; // Enable CLK Source A
    // Configure PWM0
    pwm0_do_config();
}

//---------------------------------------------------------------------
// Start PWM
//---------------------------------------------------------------------
void Start_PWM(void)
{
    // Start PWM0
    PWMCNTS = 0x01; // (0000_0001) Start  CLKA
}


//---------------------------------------------------------------------
// Start DSP (Unit0,Unit1)
//---------------------------------------------------------------------
void DSP_start(void)
{
    DSP0CTRL |= 0x80;   //DSPE=1 : DSP0 Enable
    DSP1CTRL |= 0x80;   //DSPE=1 : DSP1 Enable
}

//---------------------------------------------------------------------
// Stop DSP (Unit0,Unit1)
//---------------------------------------------------------------------
void DSP_stop(void)
{
    DSP0CTRL |= 0x00;   //DSPE=0 : DSP0 Disable
    DSP1CTRL |= 0x00;   //DSPE=0 : DSP1 Disable
}


//---------------------------------------------------------------------
// Toggle GPIO16
//---------------------------------------------------------------------
void Toggle_GPIO16(void)
{
    PDR1 = PDR1 ^ 0x40;  //[0x98]
}

//---------------------------------------------------------------------
// Toggle GPIO17
//---------------------------------------------------------------------
void Toggle_GPIO17(void)
{
    PDR1 = PDR1 ^ 0x80;  //[0x98]
}

//=========================================================
// End of Program
//=========================================================
